pub fn error_catalog() -> Vec<CatalogErrorEntry>Expand description
Build an expanded error catalog
Returns a list of all registered error types with their variants. Transparent variants are expanded to show the actual variants from the type they forward to.
Example
ⓘ
use apollo_errors::error_catalog;
let catalog = error_catalog();
let json = serde_json::to_string_pretty(&catalog).unwrap();
println!("{}", json);